#!/usr/bin/env python


import os      # needed for file operations
import sys # Needed for handling command line parameters
import time

svgFileName = "./a.svg"
gcodeFileName = "./output.gcode"


feedRateMove = " F7000.000"
feedRateCut = " F100.000"

laserPower = "S300"

scaleF = 0.01

lineEnd = "\r\n"


driveEdgesOnly = False
engraveOnly = False

for arg in sys.argv: # Loop through all command line parameters
    if arg == "-d" or arg == "--drive-edges-only":
        driveEdgesOnly = True

    elif arg == "-e" or arg == "--engrave-only":
        engraveOnly = True

# Look if the picture file exists
if os.path.isfile(svgFileName):
    print("SVG file detected, analyzing content")

    try:


        svgFile = open(svgFileName, "r") # Open the SVG file
        gcodeFile = open(gcodeFileName, "w") # Open the gcode file
        gcodeFile.write("G1 X0.100 Y0.000 S0.0" + feedRateMove + lineEnd) # Turn laser OFF
        gcodeFile.write("G1 X0.000 Y0.000 S0.0" + feedRateMove + lineEnd) # Turn laser OFF
        gcodeFile.write("$32=1" + lineEnd) # Activate laser mode
        gcodeFile.write("M3" + lineEnd) # Set to constant laser power mode
        gcodeFile.write("$1=255" + lineEnd) # Keep stepper motors powered
        gcodeFile.write("G21" + lineEnd) # Use millimeters



        lines = svgFile.readlines()
        for line in lines:
            firstX = -9999999
            firstY = -9999999
            searchPattern = "<path fill=\"none\" stroke=\"rgb(0,0,0)\" d=\"M"
            if line.find(searchPattern) > -1:
                endOfPath = False
                linePos = line.find(searchPattern)
                line_temp = line[linePos+len(searchPattern):]
                if line_temp.find("L") > -1:
                    line_temp = line_temp[line_temp.find("L")+1:]
                else:
                    endOfPath = True
                while not endOfPath:
                    linePos = line_temp.find(searchPattern)
                    if line_temp.find(",") > -1:
                        X = line_temp[0: line_temp.find(",")]
                        print("X=", X)
                        line_temp = line_temp[line_temp.find(",")+1:]
                    else:
                        endOfPath = True

                    if line_temp.find(" ") > -1:
                        Y = line_temp[0: line_temp.find(" ")]
                        print("Y=", Y)
                        line_temp = line_temp[line_temp.find(" ")+1:]
                    else:
                        endOfPath = True

                    if not endOfPath:
                        if firstX == -9999999:
                            firstX = int(X)
                            firstY = int(Y)
                            gcodeFile.write("" + lineEnd)
                            gcodeFile.write("G1 S0 X" + str(format(firstX * scaleF, ".3f")) + " Y" + str(format(firstY * scaleF, ".3f")) + feedRateMove + lineEnd)
                        else:
                            currentX = int(X)
                            currentY = int(Y)
                            gcodeFile.write("G1 " + laserPower + " X" + str(format(currentX * scaleF, ".3f")) + " Y" + str(format(currentY * scaleF, ".3f")) + feedRateCut + lineEnd)



        gcodeFile.write("" + lineEnd)
        gcodeFile.write("G1 S0 X0.100 Y0.000" + feedRateMove + lineEnd)
        gcodeFile.write("G1 S0 X0.000 Y0.000" + feedRateMove + lineEnd)
        gcodeFile.write("$1=5" + lineEnd) # Steppers disabled after 5ms (Ortur default)

        svgFile.close()
        gcodeFile.close()


    except Exception as e:
        print("An error has happened: ", e)

    except IOError: # Print message if a file error has happened
        print("Error handling Input/Output!")

    except KeyboardInterrupt:
        # This is the code section executed after 'ctrl+c' is pressed
        # Script ends after that section.
        print ("...keyboard interrupt detected\n")


exit(0)


