#!/usr/bin/python3

# makes use of python serial. Install with:
# sudo apt-get install python3-serial
# If run on a webserver, www-data must be in group dialout to access usb port:
# sudo adduser www-data dialout


import os      # needed for file operations
import sys     # Needed for handling command line parameters
import serial
import time

FileName = "./output.gcode"
feedRate = " F7000.000"

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sub open USB (grbl)
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def open_USB():
    # Open USB port to CNC machine
    grbl = serial.Serial(
    # port='/dev/ttyUSB0', #Replace ttyS0 with ttyAM0 for Pi1,Pi2,Pi0
    port='/dev/ttyACM0', #Replace ttyS0 with ttyAM0 for Pi1,Pi2,Pi0
    baudrate = 115200,
    parity=serial.PARITY_NONE,
    stopbits=serial.STOPBITS_ONE,
    bytesize=serial.EIGHTBITS,
    timeout=1
    )
    #time.sleep(1.002)# Give port some time to really open...
    print ("Initializing grbl...")
    grbl.write("$\r\n".encode('utf-8'))
    read_temp = grbl.readline().strip() # Wait for grbl response
    while read_temp != b'':
        print(read_temp) # Debug response
        read_temp = grbl.readline().strip() # Wait for grbl response
    return grbl

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sub write line to grbl
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def write2grbl(grbl, command_line = "", wait_for_idle = 1):

    # Empty read buffer
    grbl.flushInput()
  #  while(grbl.read() != b''):
  #    time.sleep(0.002)
    grbl.flush

    #time.sleep(0.01)
    # print("Writing command: ", command_line)
    command_line = command_line + '\r' + '\n'
    grbl.write(command_line.encode('utf-8'))# Write line to USB

    read_temp = grbl.readline().strip() # Wait for grbl response
    if read_temp != b'ok' and read_temp != b'':
        print("  Debug: ",read_temp) # Debug response

    #print ("Command " + str(command_line) + ": " + str(grbl_line) + " from Arduino")

    if(wait_for_idle == 1):
      grbl_line = b""
      while(grbl_line.find(b"IDLE") == -1):
        #print(grbl_line)
        grbl_line = b""
        read_from_cnc = grbl.read(1)
        if (read_from_cnc == b''):
          grbl.write(b"?\n")# Request status report
        while(read_from_cnc.upper() != b'\n'):
          if(read_from_cnc != b'\r'):
            grbl_line = grbl_line + read_from_cnc.upper()
          read_from_cnc = grbl.read(1)
        time.sleep(0.002)

    # Empty read buffer
    grbl.flushInput()


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sub end grbl and exit
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def exitScript():
    system_call = os.system("rm progress.txt")
    write2grbl(grbl, "G1 X0.0 S0.0" + feedRate, 0) # Turn laser OFF
    write2grbl(grbl, "$1=5", 0) # Turn OFF stepper motors after 5ms idle (Ortur default)
    write2grbl(grbl, "$100=80.000", 0) # X steps per mm (Ortur default)
    write2grbl(grbl, "$101=80.000", 0) # Y steps per mm (Ortur default)
    write2grbl(grbl, "G1 X0.1 Y0.0 S0.0" + feedRate, 0) # Turn laser OFF
    write2grbl(grbl, "G1 X0.0 S0.0" + feedRate, 1) # Turn laser OFF
    time.sleep(1.5)
    f.close()
    grbl.close()
    print("Script ended")
    sys.exit(0)


system_call = os.system("rm pause.txt")
system_call = os.system("rm stop.txt")

driveEdgesOnly = False
engraveOnly = False
homeOnly = False


for arg in sys.argv: # Loop through all command line parameters
    if arg == "-d" or arg == "--drive-edges-only":
        driveEdgesOnly = True

    elif arg == "-e" or arg == "--engrave-only":
        print("engraveOnly")
        engraveOnly = True

    elif arg == "-h" or arg == "--home-only":
        homeOnly = True

progressFile = open("progress.txt", "w")
progressFile.write("Starting...")
progressFile.close()

# Look if the picture file exists
if os.path.isfile(FileName):
    print("Gcode file detected, sending content")
    grbl = open_USB()

    try:
#        if homeOnly or (not engraveOnly and not driveEdgesOnly):
#            write2grbl(grbl, "$H", 0) # Home axes

#        if homeOnly:
#            exitScript()


         # Open the gcode file
         f = open(FileName, "r")
         lines = f.readlines()
         for loopRun in range(1):
            for line in lines:

              if os.path.isfile("pause.txt"):
                  write2grbl(grbl, "G1 X0.1 S0.0" + feedRate, 0) # Turn laser OFF
                  write2grbl(grbl, "G1 X0.0 S0.0" + feedRate, 0) # Turn laser OFF
              while os.path.isfile("pause.txt"):
                  next

              if os.path.isfile("stop.txt"):
                  system_call = os.system("rm stop.txt")
                  exitScript()


              line_temp = line
              if line.find(";") > -1:
                  line_temp = line_temp[0:line.find(";") - 1]

              if line.find("G0") > -1 or line.find("G1") > -1:
                  write2grbl(grbl, line_temp, 0)
              else:
                  write2grbl(grbl, line_temp, 1)


         exitScript()

    except Exception as e:
        print("An error has happened: ", e)

    except IOError: # Print message if a file error has happened
        print("Error handling Input/Output!")

    except KeyboardInterrupt:
        # This is the code section executed after 'ctrl+c' is pressed
        # Script ends after that section.
        print ("...keyboard interrupt detected\n")
        exitScript()

print("Exit")
exit(0)


