#!/usr/bin/python3

# makes use of python serial. Install with:
# sudo apt-get install python3-serial
# If run on a webserver, www-data must be in group dialout to access usb port:
# sudo adduser www-data dialout


import os      # needed for file operations
import sys     # Needed for handling command line parameters
import serial
import time


feedRate = " F7000.000"


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sub open USB (grbl)
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def open_USB():
    # Open USB port to CNC machine
    grbl = serial.Serial(
    # port='/dev/ttyUSB0', #Replace ttyS0 with ttyAM0 for Pi1,Pi2,Pi0
    port='/dev/ttyACM0', #Replace ttyS0 with ttyAM0 for Pi1,Pi2,Pi0
    baudrate = 115200,
    parity=serial.PARITY_NONE,
    stopbits=serial.STOPBITS_ONE,
    bytesize=serial.EIGHTBITS,
    timeout=1
    )
    #time.sleep(1.002)# Give port some time to really open...
    print ("Initializing grbl...")
    grbl.write("$\r\n".encode('utf-8'))
    read_temp = grbl.readline().strip() # Wait for grbl response
    while read_temp != b'':
        print(read_temp) # Debug response
        read_temp = grbl.readline().strip() # Wait for grbl response
    return grbl

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sub write line to grbl
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def write2grbl(grbl, command_line = "", wait_for_idle = 1):

    # Empty read buffer
    grbl.flushInput()
  #  while(grbl.read() != b''):
  #    time.sleep(0.002)
    grbl.flush

    #time.sleep(0.01)
    # print("Writing command: ", command_line)
    command_line = command_line + '\r' + '\n'
    grbl.write(command_line.encode('utf-8'))# Write line to USB

    read_temp = grbl.readline().strip() # Wait for grbl response
    if read_temp != b'ok' and read_temp != b'':
        print("  Debug: ",read_temp) # Debug response

    #print ("Command " + str(command_line) + ": " + str(grbl_line) + " from Arduino")

    if(wait_for_idle == 1):
      grbl_line = b""
      while(grbl_line.find(b"IDLE") == -1):
        #print(grbl_line)
        grbl_line = b""
        read_from_cnc = grbl.read(1)
        if (read_from_cnc == b''):
          grbl.write(b"?\n")# Request status report
        while(read_from_cnc.upper() != b'\n'):
          if(read_from_cnc != b'\r'):
            grbl_line = grbl_line + read_from_cnc.upper()
          read_from_cnc = grbl.read(1)
        time.sleep(0.002)

    # Empty read buffer
    grbl.flushInput()

grbl = open_USB()

for arg in sys.argv: # Loop through all command line parameters
    if arg[0:2] == "x=" or arg[0:2] == "X=":
        print("X=", arg[2:])
        write2grbl(grbl, "G1 X" + arg[2:] + "S0.0" + feedRate, 1)
        write2grbl(grbl, chr(24), 1)

    elif arg[0:2] == "y=" or arg[0:2] == "Y=":
        print("Y=", arg[2:])
        write2grbl(grbl, "G1 Y" + arg[2:] + "S0.0" + feedRate, 1)


grbl.close()
