#!/usr/bin/python3

import os      # needed for file operations
import sys # Needed for handling command line parameters
import time

svgFileName = "./Etching.svg"
gcodeFileName = "./output.gcode"

feedRateMove = " F7000.000"
feedRateCut = " F670.000"

laserPower = "S450"

scaleF = 0.01

lineEnd = "\r\n"


driveEdgesOnly = False
engraveOnly = False

for arg in sys.argv: # Loop through all command line parameters
    if arg == "-d" or arg == "--drive-edges-only":
        driveEdgesOnly = True

    elif arg == "-e" or arg == "--engrave-only":
        engraveOnly = True

# Look if the picture file exists
if os.path.isfile(svgFileName):
    print("SVG file detected, analyzing content")

    try:


        svgFile = open(svgFileName, "r") # Open the SVG file
        gcodeFile = open(gcodeFileName, "w") # Open the gcode file
        lines = svgFile.readlines()


        minX = 9999999
        minY = 9999999
        maxX = -9999999
        maxY = -9999999

        for loopRun in range(2):

            if loopRun == 1:
                gcodeFile.write("; maxX=" + str(format((maxX - minX) * scaleF, ".3f")) + " maxY=" + str(format((maxY - minY) * scaleF, ".3f")) + lineEnd)
                gcodeFile.write("G1 X0.100 Y0.000 S0.0" + feedRateMove + lineEnd) # Turn laser OFF
                gcodeFile.write("G1 X0.000 Y0.000 S0.0" + feedRateMove + lineEnd) # Turn laser OFF
                gcodeFile.write("$32=1" + lineEnd) # Activate laser mode
                gcodeFile.write("M3" + lineEnd) # Set to constant laser power mode
                gcodeFile.write("$1=255" + lineEnd) # Keep stepper motors powered
                gcodeFile.write("G21" + lineEnd) # Use millimeters

            for line in lines:
                firstX = -9999999
                firstY = -9999999
                lastX = -9999999
                lastY = -9999999
                searchPattern = '<path fill="none" stroke="rgb(0,0,0)" d="M'
                if line.find(searchPattern) > -1:
                    endOfPath = False
                    linePos = line.find(searchPattern)
                    line_temp = line[linePos+len(searchPattern):]

                    if line_temp.find(",") > -1:
                        X = line_temp[0: line_temp.find(",")]
                        print("firstX=" + X)
                        line_temp = line_temp[line_temp.find(",")+1:]
                    else:
                        endOfPath = True

                    if line_temp.find(" ") > -1:
                        Y = line_temp[0: line_temp.find(" ")]
                        print("firstY=" + Y)
                        line_temp = line_temp[line_temp.find(" ")+1:]
                    else:
                        endOfPath = True

                    if loopRun == 1:
                        firstX = int(X) - minX
                        firstY = int(Y) - minY
                        firstY = (maxY - minY) - firstY # mirror Y axis
                        gcodeFile.write("" + lineEnd)
                        gcodeFile.write("G1 S0 X" + str(format(firstX * scaleF, ".3f")) + " Y" + str(format(firstY * scaleF, ".3f")) + feedRateMove + lineEnd)

                        #gcodeFile.write("G1 " + laserPower + " X" + str(format(firstX * scaleF+0.1, ".3f")) + " Y" + str(format(firstY * scaleF+0.1, ".3f")) + " F100.000 " + lineEnd)
                    else:
                        if int(X) < minX:
                            minX = int(X)
                        if int(Y) < minY:
                            minY = int(Y)
                        if int(X) > maxX:
                            maxX = int(X)
                        if int(Y) > maxY:
                            maxY = int(Y)

                    if line_temp.find("L") > -1:
                        line_temp = line_temp[line_temp.find("L")+1:]

                    else:
                        endOfPath = True

                    while not endOfPath:
                        if line_temp.find(",") > -1:
                            X = line_temp[0: line_temp.find(",")]
                            print("X=" + X)
                            line_temp = line_temp[line_temp.find(",")+1:]
                        else:
                            endOfPath = True

                        if line_temp.find(" ") > -1:
                            Y = line_temp[0: line_temp.find(" ")]
                            print("Y=" + Y)
                            line_temp = line_temp[line_temp.find(" ")+1:]
                        else:
                            endOfPath = True

                        if not endOfPath:
                            currentX = int(X) - minX
                            currentY = int(Y) - minY
                            currentY = (maxY - minY) - currentY # mirror Y axis
                            lastX = currentX
                            lastY = currentY
                            if loopRun == 1:
                                gcodeFile.write("G1 " + laserPower + " X" + str(format(currentX * scaleF, ".3f")) + " Y" + str(format(currentY * scaleF, ".3f")) + feedRateCut + lineEnd)
                            else:
                                if int(X) < minX:
                                    minX = int(X)
                                if int(Y) < minY:
                                    minY = int(Y)
                                if int(X) > maxX:
                                    maxX = int(X)
                                if int(Y) > maxY:
                                    maxY = int(Y)

                    if lastX > -9999999 and lastY > -9999999 and loopRun == 1:
                        lastX = lastX + 10
                        gcodeFile.write("G1 S0 " + " X" + str(format(lastX * scaleF, ".3f")) + " Y" + str(format(lastY * scaleF, ".3f")) + feedRateCut + lineEnd)


        gcodeFile.write("" + lineEnd)
        gcodeFile.write("G1 S0 X0.100 Y0.000" + feedRateMove + lineEnd)
        gcodeFile.write("G1 S0 X0.000 Y0.000" + feedRateMove + lineEnd)
        gcodeFile.write("$1=5" + lineEnd) # Steppers disabled after 5ms (Ortur default)

        svgFile.close()
        gcodeFile.close()


    except Exception as e:
        print("An error has happened: ", e)

    except IOError: # Print message if a file error has happened
        print("Error handling Input/Output!")

    except KeyboardInterrupt:
        # This is the code section executed after 'ctrl+c' is pressed
        # Script ends after that section.
        print ("...keyboard interrupt detected\n")


exit(0)


