#!/usr/bin/python3

# makes use of python serial. Install with:
# sudo apt-get install python3-serial
# If run on a webserver, www-data must be in group dialout to access usb port:
# sudo adduser www-data dialout


import os      # needed for file operations
import sys     # Needed for handling command line parameters
import serial
import time

# FileName = "./tux-caveman_02_80x80.bmp"
# FileName = "./tux-caveman_02_300x300.bmp"
# FileName = "./weilburg_02.bmp"
# FileName = "./test.bmp"
# FileName = "./test_02.bmp"
# FileName = "./black-square.bmp"
FileName = "./engrave.bmp"

bfType = ""
bfSize = 0
bfReserved = 0
bfOffBits = 0
biSize = 0
biWidth = 0
biHeight = 0
biPlanes = 0
biBitCount = 0
biCompression = 0
biSizeImage = 0
biXPelsPerMeter = 0
biYPelsPerMeter = 0
biClrImportant = 0

pixelW = 0.111
pixelH = 0.111

offsetX = 0.000
offsetY = -35.000 * 0.000 + 0.100

feedRate = " F7000.000"

# gcodeFile = open("grbl.gcode", "w")


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sub open USB (grbl)
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def open_USB():
    # Open USB port to CNC machine
    grbl = serial.Serial(
    # port='/dev/ttyUSB0', #Replace ttyS0 with ttyAM0 for Pi1,Pi2,Pi0
    port='/dev/ttyACM0', #Replace ttyS0 with ttyAM0 for Pi1,Pi2,Pi0
    baudrate = 115200,
    parity=serial.PARITY_NONE,
    stopbits=serial.STOPBITS_ONE,
    bytesize=serial.EIGHTBITS,
    timeout=1
    )
    #time.sleep(1.002)# Give port some time to really open...
    print ("Initializing grbl...")
    grbl.write("$\r\n".encode('utf-8'))
    read_temp = grbl.readline().strip() # Wait for grbl response
    while read_temp != b'':
        print(read_temp) # Debug response
        read_temp = grbl.readline().strip() # Wait for grbl response
    return grbl

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sub write line to grbl
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def write2grbl(grbl, command_line = "", wait_for_idle = 1):

    # Empty read buffer
    grbl.flushInput()
  #  while(grbl.read() != b''):
  #    time.sleep(0.002)
    grbl.flush

    #time.sleep(0.01)
    #print("Writing command: ", command_line)
    command_line = command_line + '\r' + '\n'
    grbl.write(command_line.encode('utf-8'))# Write line to USB

    read_temp = grbl.readline().strip() # Wait for grbl response
    if read_temp != b'ok' and read_temp != b'':
        print("  Debug: ",read_temp) # Debug response

    #print ("Command " + str(command_line) + ": " + str(grbl_line) + " from Arduino")

    if(wait_for_idle == 1):
      grbl_line = b""
      while(grbl_line.find(b"IDLE") == -1):
        #print(grbl_line)
        grbl_line = b""
        read_from_cnc = grbl.read(1)
        if (read_from_cnc == b''):
          grbl.write(b"?\n")# Request status report
        while(read_from_cnc.upper() != b'\n'):
          if(read_from_cnc != b'\r'):
            grbl_line = grbl_line + read_from_cnc.upper()
          read_from_cnc = grbl.read(1)
        time.sleep(0.002)

    # Empty read buffer
    grbl.flushInput()

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sub write line to gcode file
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def write2gcodeFile(grbl, command_line = "", wait_for_idle = 1):

  command_line = command_line + '\r' + '\n'
  gcodeFile.write(command_line)

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sub end grbl and exit
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def exitScript():
    system_call = os.system("rm progress.txt")
    write2grbl(grbl, "G1 X0.0 S0.0" + feedRate, 0) # Turn laser OFF
    write2grbl(grbl, "$1=5", 0) # Turn OFF stepper motors after 5ms idle (Ortur default)
    write2grbl(grbl, "$100=80.000", 0) # X steps per mm (Ortur default)
    write2grbl(grbl, "$101=80.000", 0) # Y steps per mm (Ortur default)
    write2grbl(grbl, "G1 X0.1 S0.0" + feedRate, 0) # Turn laser OFF
    write2grbl(grbl, "G1 X0.0 S0.0" + feedRate, 0) # Turn laser OFF
    write2grbl(grbl, "$1=5", 1) # Turn OFF stepper motors after 5ms idle (Ortur default)
    time.sleep(2.5)
    f.close()
    grbl.close()
    print("Script ended")
    sys.exit(0)


system_call = os.system("rm pause.txt")
system_call = os.system("rm stop.txt")

driveEdgesOnly = False
engraveOnly = False
homeOnly = False


for arg in sys.argv: # Loop through all command line parameters
    if arg == "-d" or arg == "--drive-edges-only":
        driveEdgesOnly = True
	
    elif arg == "-e" or arg == "--engrave-only":
        print("engraveOnly")
        engraveOnly = True

    elif arg == "-h" or arg == "--home-only":
        homeOnly = True

progressFile = open("progress.txt", "w")
progressFile.write("Starting...")
progressFile.close()

# Look if the picture file exists
if os.path.isfile(FileName):
    print("Picture file detected, analyzing content")
    grbl = open_USB()

    try:
        if homeOnly or (not engraveOnly and not driveEdgesOnly):
            write2grbl(grbl, "$H", 0) # Home axes
        write2grbl(grbl, "G1 X0.100 Y0.000 S0.0" + feedRate, 0) # Turn laser OFF
        write2grbl(grbl, "G1 X0.000 Y0.000 S0.0" + feedRate, 0) # Turn laser OFF
        write2grbl(grbl, "$32=1", 0) # Activate laser mode
        write2grbl(grbl, "M3", 0) # Set to constant laser power mode
        write2grbl(grbl, "$1=255", 0) # Keep stepper motors powered
        write2grbl(grbl, "G21", 0) # Use millimeters

        if homeOnly:
            exitScript()
        #write2grbl(grbl, "G1 X0.0 Y0.0 S1.0 F1", 0) # Turn laser ON to 1% power for focus adjustment
        #time.sleep(1000.0)


        # Open the picture file
        f = open(FileName, "rb")
        bfType = f.read(2) # Read file identifyer
        if str(bfType) != "b'BM'":
            print("Header >" + str(bfType) + "< Not a bitmap file!")
        else:
            print("Bitmap ID detected")

        bfSize = int.from_bytes(f.read(4), byteorder='little', signed=False)
        print("bfSize =          ", format(bfSize, '10d'))

        bfReserved = int.from_bytes(f.read(4), byteorder='little', signed=False)
        print("bfReserved =      ", format(bfReserved, '10d'))

        bfOffBits = int.from_bytes(f.read(4), byteorder='little', signed=False)
        print("bfOffBits =       ", format(bfOffBits, '10d'))

        biSize = int.from_bytes(f.read(4), byteorder='little', signed=False)
        print("biSize =          ", format(biSize, '10d'))

        biWidth = int.from_bytes(f.read(4), byteorder='little', signed=False)
        print("biWidth =         ", format(biWidth, '10d'))

        biHeight = int.from_bytes(f.read(4), byteorder='little', signed=False)
        print("biHeight =        ", format(biHeight, '10d'))

        biPlanes = int.from_bytes(f.read(2), byteorder='little', signed=False)
        print("biPlanes =        ", format(biPlanes, '10d'))

        biBitCount = int.from_bytes(f.read(2), byteorder='little', signed=False)
        print("biBitCount =      ", format(biBitCount, '10d'))

        biCompression = int.from_bytes(f.read(4), byteorder='little', signed=False)
        print("biCompression =   ", format(biCompression, '10d'))

        biSizeImage = int.from_bytes(f.read(4), byteorder='little', signed=False)
        print("biSizeImage =     ", format(biSizeImage, '10d'))

        biXPelsPerMeter = int.from_bytes(f.read(4), byteorder='little', signed=False)
        print("biXPelsPerMeter = ", format(biXPelsPerMeter, '10d'))

        biYPelsPerMeter = int.from_bytes(f.read(4), byteorder='little', signed=False)
        print("biYPelsPerMeter = ", format(biYPelsPerMeter, '10d'))

        biClrUsed = int.from_bytes(f.read(4), byteorder='little', signed=False)
        print("biClrUsed =       ", format(biClrUsed, '10d'))

        biClrImportant = int.from_bytes(f.read(4), byteorder='little', signed=False)
        print("biClrImportant =  ", format(biClrImportant, '10d'))

        skipBytes = 0
        for i in range(bfOffBits - 54):# Skip bytes until bitmap data starts
            skipBytes = skipBytes + 1
            readByte = f.read(1)

        print("skipBytes =       ", format(skipBytes, '10d'))

        byteWidth = biWidth * biBitCount / 8
        while byteWidth % 4 != 0: # make byteWidth an integer of 4
            byteWidth = byteWidth + 1
        print("byteWidth =       ", format(int(byteWidth), '10d'))
        print("Empty bytes per line = ", int(byteWidth-biWidth*biBitCount/8))
        # Bitmap data starts here

        if not engraveOnly:
            answer = ""
            while answer != "y" and answer != "Y":
                write2grbl(grbl, "G1 X" + str(format(offsetX, ".3f")) + " Y"  + str(format(offsetY, ".3f")) + feedRate, 1)
                write2grbl(grbl, "G1 S9 X" + str(format(offsetX, ".3f")) + " Y" + str(biHeight * pixelH + offsetY) + feedRate, 1)
                write2grbl(grbl, "G1 S9 X" + str(biWidth * pixelW + offsetX) + " Y" + str(biHeight * pixelH + offsetY) + feedRate, 1)
                write2grbl(grbl, "G1 S9 X" + str(biWidth * pixelW + offsetX) + " Y"  + str(format(offsetY, ".3f")) + feedRate, 1)
                write2grbl(grbl, "G1 S9 X" + str(format(offsetX, ".3f")) + " Y"  + str(format(offsetY, ".3f")) + feedRate, 1)
                if driveEdgesOnly:
                    time.sleep(0.2)
                    exitScript()
                # answer = input("Workpiece placed correctly (Y/N)?")
                answer = "y" # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#        for i in range(100):
#            print ("X="+str(round(X,4)), "  ", X)
#            X = X + pixelW
#        exitScript()

        laserPower = 300
        for currentPixelY in range(biHeight):
#            if currentPixelY == 1:
#                currentPixelY = biHeight - 1
            progressFile = open("progress.txt", "w")
            progressFile.write("Line " + str(currentPixelY) + " of " + str(biHeight) + " = " + str(format(100 * currentPixelY / biHeight, "5.1f")) + "%")
            progressFile.close()
            print("Current line = ", currentPixelY + 1, " / ", biHeight)
            for currentPixelX in range(biWidth):
                while os.path.isfile("pause.txt"):
                    next

                if os.path.isfile("stop.txt"):
                    system_call = os.system("rm stop.txt")
                    exitScript()

                pixelGrey = 255
                if biBitCount == 24: # True color
                    pixelR = int.from_bytes(f.read(1), byteorder='little', signed=False)
                    pixelG = int.from_bytes(f.read(1), byteorder='little', signed=False)
                    pixelB = int.from_bytes(f.read(1), byteorder='little', signed=False)
                    pixelGrey = (pixelR + pixelG + pixelB) / 3
                    #if pixelGrey > 254:
                    #    print(" ", end = '')
                    #else:
                    #    print("X", end = '')

                if biBitCount == 8: # Grey scale
                    pixelGrey = int.from_bytes(f.read(1), byteorder='little', signed=False)
                    #if pixelGrey > 128:
                    #    print(" ", end = '')
                    #else:
                    #    print("X", end = '')

                if(pixelGrey < 255):
                    #print("pixelGrey = ", pixelGrey)
                    command = "G1 X" + str(format(currentPixelX * pixelW + offsetX, ".3f")) + " Y" + str(round(currentPixelY * pixelH + offsetY, 5)) + feedRate
                    write2grbl(grbl, command, 0)
                    command = "G1 X" + str(format(currentPixelX * pixelW + offsetX + 0.05, ".3f")) + " S" + str(laserPower) + " F" + str(format(60 * (pixelGrey+1), ".3f"))
                    write2grbl(grbl, command, 0)
                    command = "G1 X" + str(format(currentPixelX * pixelW + offsetX + 0.07, ".3f")) + " S0" + feedRate
                    write2grbl(grbl, command, 0)


            #print("")
            for i in range(int(byteWidth-biWidth*biBitCount/8)):# Discard bytes
                byte_temp = f.read(1)


        print("Job done!")
        exitScript()

    except Exception as e:
        print("An error has happened: ", e)

    except IOError: # Print message if a file error has happened
        print("Error handling Input/Output!")

    except KeyboardInterrupt:
        # This is the code section executed after 'ctrl+c' is pressed
        # Script ends after that section.
        print ("...keyboard interrupt detected\n")
        exitScript()

print("Exit")
exit(0)


